<?php
/**
 * Detector library
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		3.0.0
 * @copyright	Copyright (c) 2009 Cracker Tracker Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

// Initializes all detector arrays
function initCrackerTrackerArrays () {
	// Set error_reporting
	if (isCrackerTrackerDebug()) {
		// For debugging purposes, this is fine
		@error_reporting(E_ALL | E_STRICT);
	} else {
		// No output
		@error_reporting(0);
	}

	// Base path
	$GLOBALS['ctracker_base_path'] = dirname(dirname(__FILE__));

	// Whitelist some absolute query strings (see below)
	$GLOBALS['ctracker_whitelist'] = array(
		'cmd=new',    // LinPHA
		'cmd=edit',   // LinPHA
		'cmd=lostpw', // LinPHA
	);

	// Attacks we should detect and block
	$GLOBALS['ctracker_get_blacklist'] = array(
		// SQL injections
		'union ', ' union', 'insert ',
		'select ', ' like', 'like ', 'drop ', 'update ',
		'union(', 'union=',

		// $GLOBAL/$_SERVER array elements
		'HTTP_USER_AGENT', 'HTTP_HOST', 'HTTP_PHP', '_SESSION', 'CFG_ROOT',
		'DOCUMENT_ROOT', '_SERVER',

		// Sensitive files
		'/environ', 'etc/shadow', 'etc/gshadow', 'etc/passwd', 'etc/group',
		'etc/./shadow', 'etc/./gshadow', 'etc/./passwd', 'etc/./group',
		'.htaccess', '.htpasswd', '.htgroup', '.history', 'bash_history',
		'bashrc',

		// Other Linux/FreeBSD/??? programs (sometimes with space)
		'traceroute ', 'ping ', 'bin/xterm', 'bin/./xterm', 'lsof ',
		'telnet ', 'wget ', 'bin/perl', 'bin/id', 'uname\x20', 'uname ',
		'killall', 'diff ', 'kill ', 'locate ', 'grep ', 'vi ', 'mv ',
		'rmdir ', 'mcd ', 'mrd ', 'rm ', ' mcd', ' mrd', ' rm',
		'passwd ', ' passwd', 'mdir ', ' mdir', 'cp ', ' cp',
		'esystem ', 'chr ', ' chr', 'wget ', ' wget', ' cmd',
		'cmd ', ' rush', 'rush ', ' echr', 'echr ', ' getenv',
		'getenv', 'reboot ', 'halt ', 'powerdown ',

		// Other Linux programs (+ brace)
		'locate(', 'grep(', 'kill(', 'mcd(', 'mrd(', 'rm(', 'mv(', 'rmdir(',
		'chmod(', 'chmod(', 'chown(', 'chgrp(', 'passwd(', 'vi(', 'cp(',
		'mdir(', 'system(', 'chr(', 'wget(', 'rush(', 'echr(',

		// Other Linux programs (+ equal)
		'mcd=', 'mrd=', 'chmod=', 'chr=', 'rush=', 'echr=',

		// Paths
		'/etc/', '/bin/', '/sbin/', '/self/', '/proc/', '../../','..//', '././',
		'/home/ftp', '/home/./ftp', '/home/./www', '/home/www', '/www/virtual/',
		'/www/./virtual/',

		// Uni* commands:
		'/chgrp', '/chown', '/chmod', 'chown ', 'chmod ', 'chgrp ',

		// Compiler/interpreter
		'bin/g++ ', 'bin/c++ ', 'cc ', 'bin/python', 'bin/python', 'bin/tclsh',
		'bin/tclsh', 'bin/nasm', '/perl', 'cmd.exe',
		'nc.exe', 'ftp.exe',

		// php.ini settings
		'allow_url_fopen', 'allow_url_include', 'auto_prepend_file', 'disable_functions', 'safe_mode',
		'open_basedir',

		// PHP commands/scripts
		'fopen', 'fwrite', 'phpinfo()', '\<?', '?\>', 'base64_decode', 'file_put_contents',
		'set_magic_quotes_runtime', 'set_magic_quotes_runtime', 'display_errors', 'passthru',

		// Typical PHP script remote-inclusions and typical include file names
		'.inc.php', '.lib.php', '.class.php', 'config.php', '.inc', '_php',
		'php_', 'class_', '_class.php', 'db_mysql.inc',

		// PHP arrays
		'_phplib', '__callbackparam',

		// Generic remote inclusion
		'=http://', '=https://', '=php://',
		'path=', 'sql=',
		'=%7BQUOT%7D', '=%5C', '=%22http','=%22ftp','=%22file','=%27http','=%27ftp',
		'=%27file',

		// Wrappers
		'data://', 'tcp://', 'udp://', 'raw://', 'javascript://', 'file://', 'ftp://',

		// Blocked "users"
		'nigga(', ' nigga', 'nigga ', 'starhack', 'busca',

		// Hidden HTML stuff
		' style=', 'style =',
		'overflow:auto', 'overflow: auto',
		'overflow :auto', 'overflow : auto',
		'display:hidden', 'display: hidden',
		'display :hidden', 'display : hidden',
		'height:0px', 'height: 0px','height:1px', 'height: 1px',
		'width:0px', 'width: 0px','width:1px', 'width: 1px',

		// Uncommon user websites
		'~root', '~ftp', '~nobody',

		// Windows XP (?) hacks
		'xp_enumdsn', 'xp_availablemedia', 'xp_filelist', 'xp_cmdshell',

		// Attempts to insert links into a badly secured URL
		'%3E%3C',

		// Request header being inserted
		'content-type',

		// /proc/ and other forbidden paths
		'proc/self/environ',

		// MySQL internal functions
		'name_const',

		// @TODO Misc/unsorted
		'cgi-', '.eml', '$_request', '$_get', '$request', '$get', '.system',
		'&aim', 'new_password', '&icq', '.conf', 'motd ', 'HTTP/1.',
		'window.open', 'img src', 'img src', '.jsp', 'servlet', 'org.apache',
		'wwwacl', 'server-info', 'server-status', '/servlet/con', 'http_',
		'secure_site, ok', 'chunked', '<script', 'mod_gzip_status', '.system',
		'uol.com', ',0x', '(0x',
	);

	// BLock these words found in User-Agent
	$GLOBALS['ctracker_ua_blacklist'] = array(
		// Compiler/interpreter
		'bin/g++ ', 'bin/c++ ', 'cc ', 'bin/python', 'bin/python', 'bin/tclsh',
		'bin/tclsh', 'bin/nasm', '/perl', 'cmd.exe',
		'nc.exe', 'ftp.exe', 'wget ', 'system(', 'curl ',

		// php.ini settings
		'allow_url_fopen', 'allow_url_include', 'auto_prepend_file', 'disable_functions', 'safe_mode',

		// PHP commands/scripts
		'fopen', 'fwrite', 'phpinfo()', '\<?', '?\>', 'base64_decode', 'file_put_contents',
		'set_magic_quotes_runtime', 'set_magic_quotes_runtime', 'display_errors', 'passthru',

		// Typical PHP script remote-inclusions and typical include file names
		'.inc.php', '.lib.php', '.class.php', 'config.php', '.inc', '_php',
		'php_', 'class_', '_class.php', 'db_mysql.inc',

		// PHP arrays
		'_phplib', '__callbackparam',

		// Request header being inserted
		'content-type',

		// /proc/ and other forbidden paths
		'proc/self/environ',
	);

	// Block these words found in POST requests
	$GLOBALS['ctracker_post_blacklist'] = array(
		// This line is for detecting hidden link spam in wikis, forums, guestbooks, etc.
		' style=', 'overflow:auto', 'height:1px', 'width:1px', 'display:hidden', 'style.display',

		// "Common" login names from VHCS exploiters ;-)
		'starhack', 'DeLiMehmet', 'hisset', 'Hisset', 'delimert', 'MecTruy'
	);

	// Also block these requests (mostly you don't want CONNECT to some SMTP sites)
	$GLOBALS['ctracker_blocked_requests'] = array('CONNECT' => TRUE);

	// Init more elements
	$GLOBALS['ctracker_post_track']   = '';
	$GLOBALS['ctracker_checked_get']  = '';
	$GLOBALS['ctracker_checked_post'] = '';
	$GLOBALS['ctracker_checked_ua']   = '';
}

// Checks for worms
function isCrackerTrackerWormDetected () {
	// Check against the whole list
	$GLOBALS['ctracker_checked_get'] = urldecode(str_ireplace($GLOBALS['ctracker_get_blacklist'], '*', crackerTrackerQueryString(TRUE)));
	$GLOBALS['ctracker_checked_ua']  = urldecode(str_ireplace($GLOBALS['ctracker_ua_blacklist'], '*', crackerTrackerUserAgent(TRUE)));

	/*
	 * If it differs to original and the *whole* request string is not in
	 * whitelist then blog the attempt.
	 */
	$isWorm = (
		(
			$GLOBALS['ctracker_checked_get'] != crackerTrackerQueryString(TRUE) && (!in_array(crackerTrackerQueryString(TRUE), $GLOBALS['ctracker_whitelist']))
		) || (
			$GLOBALS['ctracker_checked_ua'] != crackerTrackerUserAgent(TRUE)
		) || (
			isset($GLOBALS['ctracker_blocked_requests'][crackerTrackerRequestMethod()])
		)
	);
	//* DEBUG-DIE: */ die('isWorm='.intval($isWorm).PHP_EOL.'get='.PHP_EOL.'"'.$GLOBALS['ctracker_checked_get'].'"'.PHP_EOL.'"'.crackerTrackerQueryString().'"'.PHP_EOL.'ua='.PHP_EOL.'"'.$GLOBALS['ctracker_checked_ua'].'"'.PHP_EOL.'"'.crackerTrackerUserAgent().'"'.PHP_EOL);

	// Return it
	return $isWorm;
}

// Checks POST data
function isCrackerTrackerPostAttackDetected () {
	// Implode recursive the whole $_POST array
	$GLOBALS['ctracker_post_track'] = urldecode(implode_r('&', $_POST));

	// Check for suspicious POST data
	$GLOBALS['ctracker_checked_post'] = urldecode(str_ireplace($GLOBALS['ctracker_post_blacklist'], '*', crackerTrackerSanitize($GLOBALS['ctracker_post_track'])));

	// Is it detected?
	return ((isCrackerTrackerWormDetected()) || ($GLOBALS['ctracker_checked_post'] != crackerTrackerSanitize($GLOBALS['ctracker_post_track'])));
}

// Prepares a mail and send it out
function sendCrackerTrackerMail () {
	// Log the attack
	crackerTrackerLogAttack();

	// Mail content
	$mail = 'Attack detected:
-----------------------------------------------------
Remote-IP       : ' . determineCrackerTrackerRealRemoteAddress() . '
User-Agent      : ' . crackerTrackerUserAgent() . '
Request-string  : ' . crackerTrackerQueryString() . '
Filtered string : ' . $GLOBALS['ctracker_checked_get'] . '
Server          : ' . crackerTrackerServerName() . '
Script          : ' . crackerTrackerScriptName() . '
Referrer        : ' . crackerTrackerReferer() . '
-----------------------------------------------------
';

	// Send it out
	crackerTrackerSendMail($mail);

	// And die here
	crackerTrackerDie();
}

// Sends the ticket emails out
function sendCrackerTrackerTicketMails () {
	// Load user template
	$mail = crackerTrackerLoadEmailTemplate('user_add_ticket', $GLOBALS['ctracker_last_ticket']);

	// Send email to the user
	crackerTrackerSendMail($mail, $GLOBALS['ctracker_last_ticket']['ctracker_ticket_email'], getCrackerTrackerLocalized('user_add_ticket_subject'));

	// Load webmaster template
	$mail = crackerTrackerLoadEmailTemplate('webmaster_add_ticket', $GLOBALS['ctracker_last_ticket'], getCrackerTrackerConfig('ctracker_language'));

	// Send email to the user
	crackerTrackerSendMail($mail, NULL, getCrackerTrackerLocalized('webmaster_add_ticket_subject'));
}

// Sends a mail out
function crackerTrackerSendMail ($mail, $recipient = NULL, $subject = NULL) {
	// Construct dummy array
	$rowData = array(
		'remote_addr' => determineCrackerTrackerRealRemoteAddress(),
		'proxy_addr'  => getenv('REMOTE_ADDR'),
		'check_get'   => $GLOBALS['ctracker_checked_get'],
		'server_name' => crackerTrackerServerName()
	);

	// Only send email if not yet found
	if (!isCrackerTrackerEntryFound($rowData)) {
		// Send the email out only in non-debug mode
		if (isCrackerTrackerDebug()) {
			// Output message
			print 'Recipient=' . $recipient . '<br />Subject=' . $subject . '<br />Text=<pre>' . $mail . '</pre>';

			// All fine
			return TRUE;
		} elseif (!is_null($recipient)) {
			// Recipient specified
			return mail($recipient, $subject, $mail, $GLOBALS['ctracker_header']);
		} elseif (isset($GLOBALS['ctracker_email'])) {
			// Send it
			return mail($GLOBALS['ctracker_email'], 'CTracker: Attack detected!', $mail, $GLOBALS['ctracker_header']);
		} else {
			// Send it the deprecated way with constant
			return mail(constant('__CTRACKER_EMAIL'), 'CTracker: Attack detected!', $mail, $GLOBALS['ctracker_header']);
		}
	} elseif (isCrackerTrackerDebug()) {
		// Output message
		print 'Recipient=' . $recipient . '<br />Subject=' . $subject . '<br />Text=<pre>' . $mail . '</pre>';

		// All fine
		return TRUE;
	}
}

// Sends a detected POST attack mail
function sendCrackerTrackerPostMail () {
	// Log the attack
	crackerTrackerLogAttack();

	// Mail text
	$mail = 'POST-Attack detected:
-----------------------------------------------------
Remote-IP            : ' . determineCrackerTrackerRealRemoteAddress() . '
User-Agent           : ' . crackerTrackerUserAgent() . '
Request-string       : ' . crackerTrackerQueryString() . '
Filtered string      : ' . $GLOBALS['ctracker_checked_get'] . '
Server               : ' . crackerTrackerServerName() . '
Script               : ' . crackerTrackerScriptName() . '
Referrer             : ' . crackerTrackerReferer() . '
-----------------------------------------------------
POST string          : ' . $GLOBALS['ctracker_post_track'] . '
Filtered POST string : ' . $GLOBALS['ctracker_checked_post'] . '
-----------------------------------------------------
';

	// Send it out
	crackerTrackerSendMail($mail);

	// And die here
	crackerTrackerDie();
}

// Sleeps for a random time and aborts the script
function crackerTrackerDie () {
	// Close database link
	crackerTrackerCloseDatabaseLink();

	// Do only sleep if debug/developer mode is not enabled
	if (!isCrackerTrackerDebug()) {
		// Sleep a little to waste the attacker's time
		sleep(mt_rand(10,30));
	} // END - if

	// Bye, bye...
	if (isCrackerTrackerDebug()) {
		// With debug backtrace in debug/developer mode
		print '<pre>';
		debug_print_backtrace();
		die('</pre>');
	} else {
		// Simple die() call
		die();
	}
}

// Logs the attack attempt
function crackerTrackerLogAttack () {
	// Aquire database link
	aquireCrackerTrackerDatabaseLink();

	// By default no proxy is used
	$proxyUsed = 'N';

	// Did the attacker use a proxy?
	if (isCrackerTrackerProxyUsed()) {
		// Set it
		$proxyUsed = 'Y';
	} // END - if

	// Prepare array for database insert
	$rowData = array(
		'remote_addr'    => determineCrackerTrackerRealRemoteAddress(),
		'proxy_addr'     => getenv('REMOTE_ADDR'),
		'user_agent'     => crackerTrackerUserAgent(),
		'get_data'       => crackerTrackerQueryString(),
		'post_data'      => $GLOBALS['ctracker_post_track'],
		'check_ua'       => $GLOBALS['ctracker_checked_ua'],
		'check_get'      => $GLOBALS['ctracker_checked_get'],
		'check_post'     => $GLOBALS['ctracker_checked_post'],
		'server_name'    => crackerTrackerServerName(),
		'script_name'    => crackerTrackerScriptName(),
		'referer'        => crackerTrackerReferer(),
		'request_method' => crackerTrackerRequestMethod(),
		'proxy_used'     => $proxyUsed,
		'first_attempt'  => 'NOW()'
	);

	// Insert the array in database
	crackerTrackerInsertArray('ctracker_data', $rowData);
}

// Alerts the current user about malicious/suspicious traffic
function crackerTrackerAlertCurrentUser () {
	// Is there some data?
	if (isset($GLOBALS['ctracker_last_suspicious_entry'])) {
		// Does the user have a ticket?
		if (ifCrackerTrackerIpHasTicket()) {
			// Load "Thank you" template
			crackerTrackerLoadTemplate('add_ticket_thanks');
		} elseif ((isset($_POST['ctracker_add_ticket'])) && (!empty($_POST['name'])) && (!empty($_POST['email']))) {
			// Add the ticket
			addCrackerTrackerTicket($_POST);

			// Send the email out
			sendCrackerTrackerTicketMails();
		} else {
			// Display the form for new ticket
			crackerTrackerLoadTemplate('add_ticket');
		}
	} // END - if

	// And stop here
	die();
}
